# Generated by Django 5.1.3 on 2024-11-09 15:01

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_core', '0002_alter_user_options_alter_onetimepassword_code_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='onetimepassword',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]
