# Generated by Django 5.1.3 on 2024-11-09 14:12

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_core', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'User', 'verbose_name_plural': 'Users'},
        ),
        migrations.AlterField(
            model_name='onetimepassword',
            name='code',
            field=models.CharField(max_length=6, unique=True, validators=[django.core.validators.RegexValidator(message='Code must be 6 digits.', regex='^\\d{6}$')]),
        ),
        migrations.AlterField(
            model_name='user',
            name='email',
            field=models.EmailField(help_text='Required. Enter a valid email address.', max_length=255, unique=True, verbose_name='Email Address'),
        ),
    ]
